"""
Send message to topic Kafka, specify by topic

Usage:

>>> import socket
>>> config = {
>>>    'bootstrap.servers': "<host>:<port>",
>>>    'client.id': socket.gethostname(),
>>>    'message.max.bytes': 1024 * 1024 * 50
>>> }

>>> producer = KafkaProducer(config=config)
>>> producer.produce(msg="Hello World", topic="tests")


"""
import json
import socket
import sys
from typing import Optional
from confluent_kafka import Producer

# Create logger
from vunv79_utilities.libs.helpers.metaclass import SingletonMeta
from vunv79_utilities.libs.logger.setup import init_logging

logger = init_logging(name="kafka_producer")


class KafkaProducer(metaclass=SingletonMeta):
    def __init__(self, config, debug=False):
        self.producer = Producer(config)
        self.debug = debug
        logger.debug("Initial kafka producer")

    def acked(self, err, msg):
        if err is not None:
            logger.error("Failed to deliver message: %s: %s" % (str(msg), str(err)))
        else:
            if self.debug:
                logger.debug("Message produced: %s" % (str(msg)))

    def produce(self, msg, topic: str, key: Optional[str] = None):
        try:
            data_encode = json.dumps(msg,
                                     default=str).encode('utf-8')
            message_bytes = sys.getsizeof(data_encode)  # Get size of data
            self.producer.produce(topic=topic,
                                  key=key,
                                  value=data_encode,
                                  callback=self.acked)
            logger.info(f"Produce message successful to topic: |{topic}| - Key: {key} Bytes: {message_bytes}")
        except Exception as ex:
            logger.error(f"kafka_producer :: produce :: Ex -> {ex}")
        self.producer.poll(1)



"""
Consume message from topic specify

Usage:

>>> import socket
>>> import json
>>> CONSUMER_CONFIG = {
>>>    'bootstrap.servers': "<host>:<port>",
>>>    'group.id': "[DetectBruteForce]-save",
>>>    "client.id": socket.gethostname(),
>>>    'auto.offset.reset': 'latest'
>>>}
>>>

>>> class DetectBruteForce(BaseConsumer):
>>>    def __init__(self):
>>>        super().__init__(config=CONSUMER_CONFIG)

>>>    def process_msg(self, msg):
>>>        logger.info("Consumed event from topic |{topic}|".format(
>>>            topic=msg.topic()))
>>>        message_text = msg.value().decode('utf-8')
>>>        payload: dict = json.loads(message_text)
>>>        ... # Continue handle
"""
from confluent_kafka import Consumer


# Create logger
from vunv79_utilities.libs.logger.setup import init_logging

logger = init_logging(name="kafka_consumer")


class BaseConsumer:
    def __init__(self, config):
        self.config = config
        self._consumer = Consumer(self.config)

    @staticmethod
    def print_assignment(consumer, partitions):
        logger.info(f'Assignment: {consumer}, {partitions}')

    def process_msg(self, *args, **kwargs):
        ...

    def consume(self, topics):
        try:
            self._consumer.subscribe(topics, on_assign=self.print_assignment)
            while True:
                msg = self._consumer.poll(timeout=1.0)
                if msg is None:
                    continue

                if msg is None:
                    logger.info("Waiting...")
                elif msg.error():
                    logger.info("ERROR: %s".format(msg.error()))
                else:
                    self.process_msg(msg)

        finally:
            # Close down consumer to commit final offsets.
            self._consumer.close()



import requests

from vunv79_utilities.libs.logger.setup import init_logging
from vunv79_utilities.libs.telegram.config import HEADERS

logger = init_logging(name="telegram_channel")


class Channel:
    def __init__(self,
                 channel_id,
                 access_token,
                 parse_mode="HTML"):
        if not channel_id:
            raise Exception("Channel ID is not provide")

        # Declare properties
        self.channel_id = channel_id
        self.access_token = access_token
        self.parse_mode = parse_mode
        self.url = f"https://api.telegram.org/bot{self.access_token}/sendMessage"

    def send(self, msg: str):
        if not msg or not msg.strip():
            raise ValueError("Message is not empty")
        payload = {
            "chat_id": f"@{self.channel_id}",
            "parse_mode": self.parse_mode,
            "text": msg
        }
        try:
            response = requests.post(self.url, json=payload, headers=HEADERS)
            result = response.json()
            if result.get("ok"):
                logger.info(f"Send message Telegram channel {self.channel_id} successful")
            else:
                logger.info(result)
        except Exception as ex:
            logger.error(f"AnomalyDetectionChannel :: send :: Ex -> {ex}")



import logging

FORMAT = '%(asctime)-1s - %(name)s - [%(levelname)s]:: %(message)s'


def init_logging(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)

    formatter = logging.Formatter(FORMAT)
    ch.setFormatter(formatter)

    # add ch to logger
    logger.addHandler(ch)
    return logger



from setuptools import setup, find_packages


def readme():
    with open('./README.md', 'r') as readme:
        return readme.read()


VERSION = '1.0.3'
DESCRIPTION = 'Simply pre-written small packages that implement in code'

# Setting up
setup(
    # the name must match the folder name 'verysimplemodule'
    name="vunv79_utilities",
    version=VERSION,
    author="Vu Nguyen",
    author_email="vuviet.nguyen.it@gmail.com",
    description=DESCRIPTION,
    long_description=readme(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[],

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
